"use strict";
const test = require("tape");
const uuid = require("node-uuid")

const STANDARD_HEADER = "X-Request-ID"
const CUSTOM_HEADER = "MY-CUSTOM-HEADER"

const GUID_REGEX = /^[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/

const request = {
    _headers: {},

    header: function(name, defaultValue) {
        return this._headers[name] || defaultValue
    }
}

const response = {
    _headers: {},

    header: function(name, value) {
        if(typeof(value) === "undefined") {
            return this._headers[name]
        }

        this._headers[name] = value;

        return value;
    }
}

function reset(){
    request._headers = {};
    response._headers = {};
}

test('new request id is set when none passed using standard header', (t)=>{

    const requestId = require("./index.js")();

    requestId(request, response, ()=>{
        
        // t.ok(request.header(STANDARD_HEADER))
        const resHeader = response.header(STANDARD_HEADER) 

        t.ok(GUID_REGEX.test(resHeader))

        reset();
        t.end();
    })
})

test('new request id is set when value passed using standard header', (t)=>{

    const requestId = require("./index.js")();

    const id = uuid.v4();
    request._headers[STANDARD_HEADER] = id;

    requestId(request, response, ()=>{
        
        const resHeader = response.header(STANDARD_HEADER) 
        console.log(response.body)

        t.equal(resHeader, id);

        reset();
        t.end();
    })
})

test('new request id is set when none passed using custom header', (t)=>{

    const requestId = require("./index.js")(CUSTOM_HEADER);

    requestId(request, response, ()=>{
        
        // t.ok(request.header(STANDARD_HEADER))
        const resHeader = response.header(CUSTOM_HEADER) 

        t.ok(GUID_REGEX.test(resHeader))

        reset();
        t.end();
    })
})

test('new request id is set when value passed using custom header', (t)=>{

    const requestId = require("./index.js")(CUSTOM_HEADER);

    const id = uuid.v4();
    request._headers[CUSTOM_HEADER] = id;

    requestId(request, response, ()=>{
        
        const resHeader = response.header(CUSTOM_HEADER) 

        t.equal(resHeader, id);

        reset();
        t.end();
    })
})


test("Integration: Request header is set in respone with customer header name", function(t){
    t.plan(2);

    const requestId = require("./index.js")(CUSTOM_HEADER);

    const restify = require("restify");
    const server = restify.createServer();


    server.pre(restify.pre.userAgentConnection());

    server.use(requestId)

    server.get("/", (req, res, next)=>{
        res.send(200, {})
    })
    server.listen(0);
    server.on('listening', function() {
        const port = server.address().port;

        const client = restify.createJsonClient({url:"http://127.0.0.1:" + port});

        client.get("/", (err, req, res, obj)=>{
            t.equals(res.statusCode, 200);

            t.ok(GUID_REGEX.test(res.header(CUSTOM_HEADER)))
        });

        // give callbacks time to finish before exiting
        setTimeout(()=>{
            client.close();
            server.close();
        }, 1000);
    });

    test("Integration: Request header is set in respone with standard header name", function(t){
        t.plan(2);

        const requestId = require("./index.js")();

        const restify = require("restify");
        const server = restify.createServer();


        server.pre(restify.pre.userAgentConnection());

        server.use(requestId)

        server.get("/", (req, res, next)=>{
            res.send(200, {})
        })
        server.listen(0);
        server.on('listening', function() {
            const port = server.address().port;

            const client = restify.createJsonClient({url:"http://127.0.0.1:" + port});

            client.get("/", (err, req, res, obj)=>{
                t.equals(res.statusCode, 200);

                t.ok(GUID_REGEX.test(res.header(STANDARD_HEADER)))
            });

            // give callbacks time to finish before exiting
            setTimeout(()=>{
                client.close();
                server.close();
            }, 1000);
        });

    });

});